﻿''' <summary>
'''     Base model - we'll use integer keys
''' </summary>
Public Interface IBaseModel
    Property Id() As Integer
End Interface

''' <summary>
'''     Seems recursive but allowed to type this as a model with an identifier
''' </summary>
''' <typeparam name="T">The type that derives from base model</typeparam>
Public MustInherit Class BaseModel(Of T As BaseModel(Of T))
    Implements IBaseModel
    ''' <summary>
    '''     Standard key
    ''' </summary>
    Public Property Id() As Integer Implements IBaseModel.Id

    ''' <summary>
    '''     Make it easy for equality 
    ''' </summary>
    ''' <param name="obj"></param>
    ''' <returns></returns>
    Public Overloads Overrides Function Equals(ByVal obj As Object) As Boolean
        Return TypeOf obj Is T AndAlso (CType(obj, T)).Id.Equals(Id)
    End Function

    ''' <summary>
    '''     Hash code is the identifier
    ''' </summary>
    ''' <returns></returns>
    Public Overrides Function GetHashCode() As Integer
        Return Id
    End Function

    ''' <summary>
    '''     Be helpful for debugging
    ''' </summary>
    ''' <returns></returns>
    Public Overrides Function ToString() As String
        Return String.Format("{0} with Id {1}", GetType(T).FullName, Id)
    End Function
End Class