﻿Imports System
Imports System.Collections.Generic

''' <summary>
'''     Help with tombstoning view models
''' </summary>
Public Class TombstoneModel
    ''' <summary>
    '''     The type of the view model - can be interface, etc.
    ''' </summary>
    Public Property SyncType() As Type

    ''' <summary>
    '''     Set up the state dictionary on construction
    ''' </summary>
    Public Sub New()
        State = New Dictionary(Of String, Object)()
    End Sub

    ''' <summary>
    '''     A dictionary of states - set the key and the object 
    '''     and it's stored
    ''' </summary>
    Public Property State() As Dictionary(Of String, Object)

    ''' <summary>
    '''     Helper method to try to get a value out 
    '''     Supply a default value if the value doesn't exist
    ''' </summary>
    ''' <typeparam name="T">The type of the stored value</typeparam>
    ''' <param name="key">The key</param>
    ''' <param name="defaultValue">A default if it doesn't exist</param>
    ''' <returns>The value or the default</returns>
    Public Function TryGet(Of T)(ByVal key As String, ByVal defaultValue As T) As T
        If State.ContainsKey(key) Then
            Return CType(State(key), T)
        End If
        Return defaultValue
    End Function
End Class