﻿Imports System.Windows
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports SterlingRecipes.Contracts
Imports SterlingRecipes.Database
Imports SterlingRecipes.Models
Imports SterlingRecipes.ViewModels
Imports UltraLight.MVVM
Imports Wintellect.Sterling

Partial Public Class App
#Region "Sterling Section"

    Private Shared _engine As SterlingEngine
    Private Shared _logger As SterlingDefaultLogger

    ''' <summary>
    '''     Reference to the main database engine
    ''' </summary>
    Private Shared privateDatabase As ISterlingDatabaseInstance
    Public Shared Property Database() As ISterlingDatabaseInstance
        Get
            Return privateDatabase
        End Get
        Private Set(ByVal value As ISterlingDatabaseInstance)
            privateDatabase = value
        End Set
    End Property

    ''' <summary>
    '''     Use this is fire up the engine
    ''' </summary>
    Private Shared Sub _ActivateEngine()
        _engine = New SterlingEngine()

        ' custom serializer for types
        _engine.SterlingDatabase.RegisterSerializer(Of TypeSerializer)()

        ' change this for more verbose messages
        _logger = New SterlingDefaultLogger(SterlingLogLevel.Information)
        _engine.Activate()

        ' set it 
        Database = _engine.SterlingDatabase.RegisterDatabase(Of RecipeDatabase)()

        ' see if we need to load it 
        RecipeDatabase.CheckAndCreate(Database)
    End Sub

    ''' <summary>
    '''     This registers the view models we'll be using at runtime
    ''' </summary>
    Private Shared Sub _RegisterViewModels()
        UltraLightLocator.Register(Of IMainViewModel)(New MainViewModel())
        UltraLightLocator.Register(Of IRecipeViewModel)(New RecipeViewModel())
        UltraLightLocator.Register(Of IIngredientViewModel)(New IngredientViewModel())
        UltraLightLocator.Register(Of ITextEditorViewModel)(New TextEditorViewModel())
    End Sub

    ''' <summary>
    '''     Use this to clear the database and flush it out
    ''' </summary>
    Private Shared Sub _DeactivateEngine()
        Database.Flush()
        _logger.Detach()
        _engine.Dispose()
        Database = Nothing
        _engine = Nothing
    End Sub

#End Region

    ''' <summary>
    ''' Provides easy access to the root frame of the Phone Application.
    ''' </summary>
    Private privateRootFrame As PhoneApplicationFrame
    Public Property RootFrame() As PhoneApplicationFrame
        Get
            Return privateRootFrame
        End Get
        Private Set(ByVal value As PhoneApplicationFrame)
            privateRootFrame = value
        End Set
    End Property

    ''' <summary>
    ''' Constructor for the Application object.
    ''' </summary>
    Public Sub New()
        ' Global handler for uncaught exceptions. 
        AddHandler UnhandledException, AddressOf _ApplicationUnhandledException

        ' Show graphics profiling information while debugging.
        If System.Diagnostics.Debugger.IsAttached Then
            ' Display the current frame rate counters
            Current.Host.Settings.EnableFrameRateCounter = True

            ' Show the areas of the app that are being redrawn in each frame.
            'Application.Current.Host.Settings.EnableRedrawRegions = true;

            ' Enable non-production analysis visualization mode, 
            ' which shows areas of a page that are being GPU accelerated with a colored overlay.
            'Application.Current.Host.Settings.EnableCacheVisualization = true;
        End If

        ' Standard Silverlight initialization
        InitializeComponent()

        ' Phone-specific initialization
        _InitializePhoneApplication()
    End Sub

    ' Code to execute when the application is launching (eg, from Start)
    ' This code will not execute when the application is reactivated
    Private Sub _ApplicationLaunching(ByVal sender As Object, ByVal e As LaunchingEventArgs)
        _ActivateEngine()
        _RegisterViewModels()
    End Sub

    ' Code to execute when the application is activated (brought to foreground)
    ' This code will not execute when the application is first launched
    Private Sub _ApplicationActivated(ByVal sender As Object, ByVal e As ActivatedEventArgs)
        _ActivateEngine()
        _RegisterViewModels()
    End Sub

    ' Code to execute when the application is deactivated (sent to background)
    ' This code will not execute when the application is closing
    Private Sub _ApplicationDeactivated(ByVal sender As Object, ByVal e As DeactivatedEventArgs)
        _DeactivateEngine()
    End Sub

    ' Code to execute when the application is closing (eg, user hit Back)
    ' This code will not execute when the application is deactivated
    Private Sub _ApplicationClosing(ByVal sender As Object, ByVal e As ClosingEventArgs)
        Database.Truncate(GetType(TombstoneModel))
        _DeactivateEngine()
        ' Ensure that required application state is persisted here.
    End Sub

    ' Code to execute if a navigation fails
    Private Shared Sub _RootFrameNavigationFailed(ByVal sender As Object, ByVal e As NavigationFailedEventArgs)
        If System.Diagnostics.Debugger.IsAttached Then
            ' A navigation has failed; break into the debugger
            System.Diagnostics.Debugger.Break()
        End If
    End Sub

    ' Code to execute on Unhandled Exceptions
    Private Shared Sub _ApplicationUnhandledException(ByVal sender As Object, ByVal e As ApplicationUnhandledExceptionEventArgs)
        If System.Diagnostics.Debugger.IsAttached Then
            ' An unhandled exception has occurred; break into the debugger
            System.Diagnostics.Debugger.Break()
        End If
    End Sub

#Region "Phone application initialization"

    ' Avoid double-initialization
    Private _phoneApplicationInitialized As Boolean

    Shared Sub New()
        Database = Nothing
    End Sub

    ' Do not add any additional code to this method
    Private Sub _InitializePhoneApplication()
        If _phoneApplicationInitialized Then
            Return
        End If

        ' Create the frame but don't set it as RootVisual yet; this allows the splash
        ' screen to remain active until the application is ready to render.
        RootFrame = New PhoneApplicationFrame()
        AddHandler RootFrame.Navigated, AddressOf _CompleteInitializePhoneApplication

        ' Handle navigation failures
        AddHandler RootFrame.NavigationFailed, AddressOf _RootFrameNavigationFailed

        ' Ensure we don't initialize again
        _phoneApplicationInitialized = True
    End Sub

    ' Do not add any additional code to this method
    Private Sub _CompleteInitializePhoneApplication(ByVal sender As Object, ByVal e As NavigationEventArgs)
        ' Set the root visual to allow the application to render
        RootVisual = RootFrame

        ' Remove this handler since it is no longer needed
        RemoveHandler RootFrame.Navigated, AddressOf _CompleteInitializePhoneApplication
    End Sub

#End Region
End Class