﻿'INSTANT VB NOTE: This code snippet uses implicit typing. You will need to set 'Option Infer On' in the VB file or set 'Option Infer' at the project level.

Imports System
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.Linq
Imports SterlingRecipes.Contracts
Imports SterlingRecipes.Models
Imports UltraLight.MVVM.Contracts

Namespace SampleData
    ''' <summary>
    '''     Design data for main view model
    ''' </summary>
    Public Class DesignMainViewModel
        Implements IMainViewModel
        Public Shared ReadOnly CategoryData As New List(Of CategoryModel)() From {
         New CategoryModel With {.Id = 1, .CategoryName = "Breakfast"},
         New CategoryModel With {.Id = 1, .CategoryName = "Lunch"}}

        Public Sub New()
            Categories = New ObservableCollection(Of CategoryModel)()
            For Each category In CategoryData
                Categories.Add(category)
            Next category
        End Sub

        ''' <summary>
        '''     Back request
        ''' </summary>
        ''' <returns>Used to process a back request. Return true to cancel.</returns>
        Public Function CancelBackRequest() As Boolean Implements IViewModel.CancelBackRequest
            Return False
        End Function

        ''' <summary>
        '''     To to visual state 
        ''' </summary>
        Public Property GoToVisualState() As Action(Of String, Boolean) Implements IViewModel.GoToVisualState

        ''' <summary>
        ''' A collection for ItemViewModel objects.
        ''' </summary>
        Private privateCategories As ObservableCollection(Of CategoryModel)
        Public Property Categories() As ObservableCollection(Of CategoryModel)
            Get
                Return privateCategories
            End Get
            Set(ByVal value As ObservableCollection(Of CategoryModel))
                privateCategories = value
            End Set
        End Property

        Public ReadOnly Property CategoriesAdapter() As ObservableCollection(Of CategoryModel) Implements IMainViewModel.Categories
            Get
                Return privateCategories
            End Get
        End Property

        Public ReadOnly Property CurrentCategory() As CategoryModel Implements IMainViewModel.CurrentCategory
            Get
                Return Categories(1)
            End Get
        End Property

        Public ReadOnly Property Recipes() As IEnumerable(Of RecipeModel) Implements IMainViewModel.Recipes
            Get
                Return New List(Of RecipeModel) From {
                 New RecipeModel With {.Id = 1, .Name = "Grandma's Apple Cobbler"},
                 New RecipeModel With {.Id = 2, .Name = "Mama's Chicken Noodle Soup"}}
            End Get
        End Property

        Public ReadOnly Property CurrentRecipe() As RecipeModel Implements IMainViewModel.CurrentRecipe
            Get
                Return Recipes.First()
            End Get
        End Property

        Public ReadOnly Property IsDataLoaded() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Sub LoadData()
            Return
        End Sub

        Public ReadOnly Property EditRecipe() As IActionCommand(Of Object) Implements IMainViewModel.EditRecipe
            Get
                Return Nothing
            End Get
        End Property

        Public ReadOnly Property AddRecipe() As IActionCommand(Of Object) Implements IMainViewModel.AddRecipe
            Get
                Return Nothing
            End Get
        End Property

        Public ReadOnly Property DeleteRecipe() As IActionCommand(Of Object) Implements IMainViewModel.DeleteRecipe
            Get
                Return Nothing
            End Get
        End Property

        Public Property GoBack() As Action Implements IViewModel.GoBack

        Public Property Dialog() As IDialog Implements IViewModel.Dialog
        Public Property RequestNavigation() As Action(Of String) Implements IViewModel.RequestNavigation
        Public ReadOnly Property ApplicationBindings() As IEnumerable(Of IActionCommand(Of Object)) Implements IViewModel.ApplicationBindings
            Get
                Return Nothing
            End Get
        End Property

    End Class
End Namespace