﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Threading
Imports SterlingRecipes.Contracts
Imports SterlingRecipes.Database
Imports SterlingRecipes.Messages
Imports SterlingRecipes.Models
Imports UltraLight.MVVM
Imports UltraLight.MVVM.Contracts

Namespace ViewModels
    ''' <summary>
    '''     Ingredient view model - handles editing/deleting ingredients
    ''' </summary>
    Public Class IngredientViewModel
        Inherits BaseViewModel
        Implements IIngredientViewModel, ITombstoneFriendly
        Private Const TAP_TO_SELECT As String = "(Tap to Select)"
        Private Const INGREDIENT_ID As String = "IngredientId"

        Private _ingredientId As Integer
        Private Shared _newId As Integer = -100
        'INSTANT VB NOTE: The variable _foodSearch was renamed since Visual Basic does not allow class members with the same name:
        Private _foodSearch_Renamed As Boolean

        ''' <summary>
        '''     Food search
        ''' </summary>
        Private _foodText As String

        Public Property FoodText() As String
            Get
                Return _foodText
            End Get
            Set(ByVal value As String)
                _foodText = value
                RaisePropertyChanged(Function() FoodText)
                RaisePropertyChanged(Function() Food)
                AddFood.RaiseCanExecuteChanged()
            End Set
        End Property

        Public ReadOnly Property FoodTextAdapter() As String Implements IIngredientViewModel.FoodText
            Get
                Return _foodText
            End Get
        End Property

        ''' <summary>
        '''     Title (add/edit)
        ''' </summary>
        Private _title As String

        Public Property Title() As String
            Get
                Return _title
            End Get
            Set(ByVal value As String)
                _title = value
                RaisePropertyChanged(Function() Title)
            End Set
        End Property

        Public ReadOnly Property TitleAdapter() As String Implements IIngredientViewModel.Title
            Get
                Return _title
            End Get
        End Property

        ''' <summary>
        '''     Units
        ''' </summary>
        Private _units As String

        Public Property Units() As String
            Get
                Return _units
            End Get
            Set(ByVal value As String)
                _units = value
                RaisePropertyChanged(Function() Units)
            End Set
        End Property

        Public ReadOnly Property UnitsAdapter() As String Implements IIngredientViewModel.Units
            Get
                Return _units
            End Get
        End Property

        ''' <summary>
        '''     Food filter based on entered text
        ''' </summary>
        Public ReadOnly Property Food() As IEnumerable(Of FoodModel) Implements IIngredientViewModel.Food
            Get
                If String.IsNullOrEmpty(_foodText) Then
                    Return Enumerable.Empty(Of FoodModel)()
                End If
                Dim foodTextLower = _foodText.ToLower()
                Return From f In App.Database.Query(Of FoodModel, String, Integer)(RecipeDatabase.IDX_FOOD_NAME)
                       Where f.Index.ToLower().Contains(foodTextLower)
                       Order By f.Index
                       Select New FoodModel With {.Id = f.Key, .FoodName = f.Index}
            End Get
        End Property

        ''' <summary>
        '''     Food item selected
        ''' </summary>
        Private _selectedFood As FoodModel

        Public Property SelectedFood() As FoodModel
            Get
                Return _selectedFood
            End Get
            Set(ByVal value As FoodModel)
                _selectedFood = value
                RaisePropertyChanged(Function() SelectedFood)

                If value Is Nothing Then
                    Return
                End If

                ' if not null, set the text to the selected item

                FoodText = value.FoodName

                If Not _foodSearch_Renamed Then
                    Return
                End If

                ' if in the food search, we've selected something so go back

                GoBack()()
            End Set
        End Property

        Public ReadOnly Property SelectedFoodAdapter() As FoodModel Implements IIngredientViewModel.SelectedFood
            Get
                Return _selectedFood
            End Get
        End Property

        ''' <summary>
        '''     List of measurements - using the query ensures they are cached after first load
        ''' </summary>
        Public ReadOnly Property Measures() As IEnumerable(Of MeasureModel) Implements IIngredientViewModel.Measures
            Get
                Return From m In App.Database.Query(Of MeasureModel, Integer)()
                       Order By m.LazyValue.Value.FullMeasure
                       Select m.LazyValue.Value
            End Get
        End Property

        ''' <summary>
        '''     Selected measure
        ''' </summary>
        Private _selectedMeasure As MeasureModel

        Public Property SelectedMeasure() As MeasureModel
            Get
                Return _selectedMeasure
            End Get
            Set(ByVal value As MeasureModel)
                _selectedMeasure = value
                RaisePropertyChanged(Function() SelectedMeasure)
            End Set
        End Property

        Public ReadOnly Property SelectedMeasureAdapter() As MeasureModel Implements IIngredientViewModel.SelectedMeasure
            Get
                Return _selectedMeasure
            End Get
        End Property

        ''' <summary>
        '''  Save an ingredient (update/add)
        ''' </summary>
        Private privateSaveIngredient As IActionCommand(Of Object)
        Public Property SaveIngredient() As IActionCommand(Of Object)
            Get
                Return privateSaveIngredient
            End Get
            Private Set(ByVal value As IActionCommand(Of Object))
                privateSaveIngredient = value
            End Set
        End Property

        Public ReadOnly Property SaveIngredientAdapter() As IActionCommand(Of Object) Implements IIngredientViewModel.SaveIngredient
            Get
                Return privateSaveIngredient
            End Get
        End Property

        ''' <summary>
        '''     Cancel transaction
        ''' </summary>
        Private privateCancel As IActionCommand(Of Object)
        Public Property Cancel() As IActionCommand(Of Object)
            Get
                Return privateCancel
            End Get
            Private Set(ByVal value As IActionCommand(Of Object))
                privateCancel = value
            End Set
        End Property

        Public ReadOnly Property CancelAdapter() As IActionCommand(Of Object) Implements IIngredientViewModel.Cancel
            Get
                Return privateCancel
            End Get
        End Property

        ''' <summary>
        '''     Add a new food
        ''' </summary>
        Private privateAddFood As IActionCommand(Of Object)
        Public Property AddFood() As IActionCommand(Of Object)
            Get
                Return privateAddFood
            End Get
            Private Set(ByVal value As IActionCommand(Of Object))
                privateAddFood = value
            End Set
        End Property

        Public ReadOnly Property AddFoodAdapter() As IActionCommand(Of Object) Implements IIngredientViewModel.AddFood
            Get
                Return privateAddFood
            End Get
        End Property

        ''' <summary>
        '''     Start a food search
        ''' </summary>
        Private privateFoodSearch As IActionCommand(Of Object)
        Public Property FoodSearch() As IActionCommand(Of Object)
            Get
                Return privateFoodSearch
            End Get
            Private Set(ByVal value As IActionCommand(Of Object))
                privateFoodSearch = value
            End Set
        End Property

        Public ReadOnly Property FoodSearchAdapter() As IActionCommand(Of Object) Implements IIngredientViewModel.FoodSearch
            Get
                Return privateFoodSearch
            End Get
        End Property

        ''' <summary>
        '''     Option buttons
        ''' </summary>
        Public Overrides ReadOnly Property ApplicationBindings() As IEnumerable(Of IActionCommand(Of Object))
            Get
                Return {SaveIngredient, Cancel}
            End Get
        End Property

        ''' <summary>
        '''     Set it all up
        ''' </summary>
        Public Sub New()
            SaveIngredient = New ActionCommand(Of Object)(Sub(o) _SaveIngredient())
            AddFood = New ActionCommand(Of Object)(Sub(o) _AddFood(), Function(o) (Not String.IsNullOrEmpty(FoodText)))
            Cancel = New ActionCommand(Of Object)(Sub(o) _Cancel())
            FoodSearch = New ActionCommand(Of Object)(Sub(o) _FoodSearch())
        End Sub

        ''' <summary>
        '''     Requested a food search, set the flag (we use the same view model) and then navigate
        ''' </summary>
        Private Sub _FoodSearch()
            _foodSearch_Renamed = True
            _foodText = String.Empty
            RaisePropertyChanged(Function() FoodText)
            RequestNavigation()("/Views/FoodSearch.xaml")
        End Sub

        ''' <summary>
        '''     Add a new food - make sure it doesn't exist and then save it, trigger will
        '''     auto-generate the id
        ''' </summary>
        Private Sub _AddFood()
            Dim exists = (
                From f In App.Database.Query(Of FoodModel, String, Integer)(RecipeDatabase.IDX_FOOD_NAME)
                Where f.Index.ToLower().Equals(FoodText.ToLower())
                Select f).FirstOrDefault()

            If exists IsNot Nothing Then
                SelectedFood = exists.LazyValue.Value
                Return
            End If

            Dim food = New FoodModel With {.FoodName = FoodText}
            App.Database.Save(food)
            App.Database.Flush()
            SelectedFood = food
        End Sub

        Public Overrides Function CancelBackRequest() As Boolean
            _Cancel()
            Return False
        End Function

        ''' <summary>
        '''     Cancel - go back
        ''' </summary>
        Private Sub _Cancel()
            App.Database.Delete(GetType(TombstoneModel), GetType(IIngredientViewModel))
            _ingredientId = -1
            GoBack()()
        End Sub

        ''' <summary>
        '''     Save the new ingredient
        ''' </summary>
        Private Sub _SaveIngredient()
            Const CANNOT_SAVE As String = "Cannot Save"

            Dim units As Double

            If Not Double.TryParse(Me.Units, units) Then
                Dialog.ShowMessage(CANNOT_SAVE, "Units must be a valid number.", False)
                Return
            End If

            If units <= 0 Then
                Dialog.ShowMessage(CANNOT_SAVE, "Units must be greater than 0.", False)
                Return
            End If

            If SelectedFood Is Nothing Then
                Dialog.ShowMessage(CANNOT_SAVE, "You must select a food.", False)
                Return
            End If

            ' set up a negative id to keep track of new ingredients
            If _ingredientId < 1 Then
                _ingredientId = _newId
                Interlocked.Decrement(_newId)
            End If

            ' set this up to save it
            Dim ingredient = New IngredientModel With {.Id = _ingredientId, .Amount = New MeasureAmountModel With {.Measure = App.Database.Load(Of MeasureModel)(SelectedMeasure.Id), .Units = units}, .Food = App.Database.Load(Of FoodModel)(SelectedFood.Id)}

            UltraLightLocator.EventAggregator.Publish(IngredientMessage.Create(IngredientAction.Save, ingredient))
            App.Database.Delete(GetType(TombstoneModel), GetType(IIngredientViewModel))
            GoBack()()
        End Sub

        ''' <summary>
        '''  Begin an edit transaction
        ''' </summary>
        ''' <param name="ingredientModel">The ingredient to edit</param>
        Public Sub EditIngredient(ByVal ingredientModel As IngredientModel) Implements IIngredientViewModel.EditIngredient
            App.Database.Delete(GetType(TombstoneModel), GetType(IIngredientViewModel))
            _ingredientId = ingredientModel.Id
            Title = "Edit Ingredient"
            FoodText = ingredientModel.Food.FoodName
            SelectedFood = App.Database.Load(Of FoodModel)(ingredientModel.Food.Id)
            Units = ingredientModel.Amount.Units.ToString()
            SelectedMeasure = App.Database.Load(Of MeasureModel)(ingredientModel.Amount.Measure.Id)
        End Sub

        ''' <summary>
        '''  Add a new ingredient
        ''' </summary>
        Public Sub AddIngredient() Implements IIngredientViewModel.AddIngredient
            App.Database.Delete(GetType(TombstoneModel), GetType(IIngredientViewModel))
            _ingredientId = -1
            Title = "Add Ingredient"
            FoodText = TAP_TO_SELECT
            SelectedFood = Nothing
            Units = "0"
            SelectedMeasure = Measures.First()
        End Sub


        ''' <summary>
        '''     Tombstone
        ''' </summary>
        Public Sub Deactivate() Implements ITombstoneFriendly.Deactivate
            Dim tombstone = New TombstoneModel With {.SyncType = GetType(IIngredientViewModel)}
            tombstone.State.Add(ExtractPropertyName(Function() Title), Title)
            tombstone.State.Add(ExtractPropertyName(Function() FoodText), FoodText)
            tombstone.State.Add(ExtractPropertyName(Function() Units), Units)
            tombstone.State.Add(ExtractPropertyName(Function() SelectedMeasure), If(SelectedMeasure Is Nothing, -1, SelectedMeasure.Id))
            tombstone.State.Add(ExtractPropertyName(Function() SelectedFood), If(SelectedFood Is Nothing, -1, SelectedFood.Id))
            tombstone.State.Add(INGREDIENT_ID, _ingredientId)
            App.Database.Save(tombstone)
        End Sub

        ''' <summary>
        '''     Returned from tombstone
        ''' </summary>
        Public Sub Activate() Implements ITombstoneFriendly.Activate
            Dim tombstone = App.Database.Load(Of TombstoneModel)(GetType(IIngredientViewModel))
            If tombstone Is Nothing Then
                Return
            End If
            Title = tombstone.TryGet(ExtractPropertyName(Function() Title), String.Empty)
            Units = tombstone.TryGet(ExtractPropertyName(Function() Units), "0")
            _ingredientId = tombstone.TryGet(INGREDIENT_ID, -1)
            Dim selectedMeasureId = tombstone.TryGet(ExtractPropertyName(Function() SelectedMeasure), -1)
            If selectedMeasureId > 0 Then
                SelectedMeasure = (
                    From m In Measures
                    Where m.Id = selectedMeasureId
                    Select m).FirstOrDefault()
            End If

            If _foodSearch_Renamed Then
                _foodSearch_Renamed = False
                If SelectedFood IsNot Nothing Then
                    FoodText = SelectedFood.FoodName
                End If
            Else
                FoodText = tombstone.TryGet(ExtractPropertyName(Function() FoodText), TAP_TO_SELECT)
                Dim selectedFoodId = tombstone.TryGet(ExtractPropertyName(Function() SelectedFood), -1)
                If selectedFoodId > 0 Then
                    SelectedFood = App.Database.Load(Of FoodModel)(selectedFoodId)
                End If
            End If
            If String.IsNullOrEmpty(_foodText) Then
                FoodText = TAP_TO_SELECT
            End If
        End Sub
    End Class
End Namespace