﻿Imports System.Collections.Generic
Imports SterlingRecipes.Contracts
Imports SterlingRecipes.Models
Imports UltraLight.MVVM
Imports UltraLight.MVVM.Contracts

Namespace ViewModels
    ''' <summary>
    '''     Allows a generic text edit dialog
    ''' </summary>
    Public Class TextEditorViewModel
        Inherits BaseViewModel
        Implements ITextEditorViewModel, ITombstoneFriendly
        ''' <summary>
        '''     Set it up
        ''' </summary>
        Public Sub New()
            OKCommand = New ActionCommand(Of Object)(Sub(o)
                                                         App.Database.Delete(GetType(TombstoneModel), GetType(ITextEditorViewModel))
                                                         UltraLightLocator.EventAggregator.Publish(Of ITextEditorViewModel)(Me)
                                                         GoBack()()
                                                     End Sub, Function(o) Not String.IsNullOrEmpty(Text))

            CancelCommand = New ActionCommand(Of Object)(Sub(o) _CheckCancel())
        End Sub

        ''' <summary>
        '''     For pages, the list of commands to bind to application bar buttons
        ''' </summary>
        Public Overrides ReadOnly Property ApplicationBindings() As IEnumerable(Of IActionCommand(Of Object))
            Get
                Return {OKCommand, CancelCommand}
            End Get
        End Property

        ''' <summary>
        '''     Title (pass whatever to prompt what is being edited)
        ''' </summary>
        Private _title As String

        Public Property Title() As String
            Get
                Return _title
            End Get
            Set(ByVal value As String)
                _title = value
                RaisePropertyChanged(Function() Title)
            End Set
        End Property

        Public ReadOnly Property TitleAdapter() As String Implements ITextEditorViewModel.Title
            Get
                Return _title
            End Get
        End Property

        ''' <summary>
        '''     Text to edit
        ''' </summary>
        Private _text As String

        Public Property Text() As String
            Get
                Return _text
            End Get
            Set(ByVal value As String)
                _text = value
                RaisePropertyChanged(Function() Text)
                OKCommand.RaiseCanExecuteChanged()
            End Set
        End Property

        Public ReadOnly Property TextAdapter() As String Implements ITextEditorViewModel.Text
            Get
                Return _text
            End Get
        End Property

        ''' <summary>
        '''  OK - save it
        ''' </summary>
        Private privateOKCommand As IActionCommand(Of Object)
        Public Property OKCommand() As IActionCommand(Of Object)
            Get
                Return privateOKCommand
            End Get
            Private Set(ByVal value As IActionCommand(Of Object))
                privateOKCommand = value
            End Set
        End Property

        ''' <summary>
        '''     Nope, cancel
        ''' </summary>
        Private privateCancelCommand As IActionCommand(Of Object)
        Public Property CancelCommand() As IActionCommand(Of Object)
            Get
                Return privateCancelCommand
            End Get
            Private Set(ByVal value As IActionCommand(Of Object))
                privateCancelCommand = value
            End Set
        End Property

        Public Overrides Function CancelBackRequest() As Boolean
            If Not Dialog.ShowMessage("Confirm Cancel", "Are you sure you wish to cancel? All changes will be lost!", True) Then
                Return True
            End If
            App.Database.Delete(GetType(TombstoneModel), GetType(ITextEditorViewModel))
            Return False
        End Function

        ''' <summary>
        '''     Make sure they want to cancel
        ''' </summary>
        Private Sub _CheckCancel()
            If Not Dialog.ShowMessage("Confirm Cancel", "Are you sure you wish to cancel? All changes will be lost!", True) Then
                Return
            End If
            App.Database.Delete(GetType(TombstoneModel), GetType(ITextEditorViewModel))
            GoBack()()
        End Sub

        ''' <summary>
        '''     Begin the edit stage
        ''' </summary>
        ''' <param name="title">Title to show</param>
        ''' <param name="text">Existing text</param>
        Public Sub BeginEdit(ByVal title As String, ByVal text As String) Implements ITextEditorViewModel.BeginEdit
            App.Database.Delete(GetType(TombstoneModel), GetType(ITextEditorViewModel))
            Me.Title = title
            Me.Text = text
        End Sub

        ''' <summary>
        '''     Tombstone
        ''' </summary>
        Public Sub Deactivate() Implements ITombstoneFriendly.Deactivate
            Dim tombstone = New TombstoneModel With {.SyncType = GetType(ITextEditorViewModel)}
            tombstone.State.Add(ExtractPropertyName(Function() Title), Title)
            tombstone.State.Add(ExtractPropertyName(Function() Text), Text)
            App.Database.Save(tombstone)
        End Sub

        ''' <summary>
        '''     Returned from tombstone
        ''' </summary>
        Public Sub Activate() Implements ITombstoneFriendly.Activate
            Dim tombstone = App.Database.Load(Of TombstoneModel)(GetType(ITextEditorViewModel))
            If tombstone Is Nothing Then
                Return
            End If
            Title = tombstone.TryGet(ExtractPropertyName(Function() Title), String.Empty)
            Text = tombstone.TryGet(ExtractPropertyName(Function() Text), String.Empty)
        End Sub
    End Class
End Namespace