﻿'INSTANT VB NOTE: This code snippet uses implicit typing. You will need to set 'Option Infer On' in the VB file or set 'Option Infer' at the project level.

Imports System
Imports UltraLight.MVVM.Contracts

''' <summary>
'''     Delegate command - does it all
''' </summary>
Public Class ActionCommand(Of T)
    Implements IActionCommand(Of T)
    Private _execute As Action(Of T) = Sub(obj)
                                       End Sub
    Private ReadOnly _canExecute As Func(Of T, Boolean) = Function(obj) True

    Public Property Overridden() As Boolean Implements IActionCommand.Overridden

    Public Sub New()

    End Sub

    ''' <summary>
    '''     Override the action
    ''' </summary>
    ''' <param name="action"></param>
    Public Sub OverrideAction(ByVal action As Action(Of T)) Implements IActionCommand(Of T).OverrideAction
        _execute = action
        Overridden = True
    End Sub

    ''' <summary>
    '''     Constructor with action to perform
    ''' </summary>
    ''' <param name="execute">The action to execute</param>
    Public Sub New(ByVal execute As Action(Of T))
        _execute = execute
    End Sub

    ''' <summary>
    '''     Constructor with action and condition
    ''' </summary>
    ''' <param name="execute"></param>
    ''' <param name="canExecute"></param>
    Public Sub New(ByVal execute As Action(Of T), ByVal canExecute As Func(Of T, Boolean))
        _execute = execute
        _canExecute = canExecute
    End Sub

    ''' <summary>
    ''' Defines the method that determines whether the command can execute in its current state.
    ''' </summary>
    ''' <returns>
    ''' true if this command can be executed; otherwise, false.
    ''' </returns>
    ''' <param name="parameter">Data used by the command. If the command does not require data to be passed, this object can be set to null. </param>
    Public Function CanExecute(ByVal parameter As Object) As Boolean Implements ICommand.CanExecute
        Return _canExecute(CType(parameter, T))
    End Function

    ''' <summary>
    ''' Defines the method to be called when the command is invoked.
    ''' </summary>
    ''' <param name="parameter">Data used by the command. If the command does not require data to be passed, this object can be set to null. </param>
    Public Sub Execute(ByVal parameter As Object) Implements ICommand.Execute
        If CanExecute(parameter) Then
            _execute(CType(parameter, T))
        End If
    End Sub

    Public Sub RaiseCanExecuteChanged() Implements IActionCommand.RaiseCanExecuteChanged
        Dim handler = CanExecuteChangedEvent
        If handler IsNot Nothing Then
            handler(Me, EventArgs.Empty)
        End If
    End Sub

    Public Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged
End Class