﻿Imports System.Windows
Imports System.Windows.Controls
Imports UltraLight.MVVM.Contracts

''' <summary>
'''     Button command implementation
''' </summary>
Public NotInheritable Class ButtonCommand
    Public Shared CommandProperty As DependencyProperty = DependencyProperty.RegisterAttached("Command", GetType(IActionCommand(Of Object)), GetType(ButtonCommand), New PropertyMetadata(Nothing, AddressOf CommandChanged))

    ''' <summary>
    '''     Attach to the button and handle click and is enabled
    ''' </summary>
    Private Sub New()
    End Sub
    Private Shared Sub CommandChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
        Dim button = TryCast(d, Button)
        Dim command = TryCast(e.NewValue, IActionCommand(Of Object))
        If d Is Nothing OrElse command Is Nothing Then
            Return
        End If
        AddHandler button.Click, Sub(o, args) command.Execute(GetCommandParameter(CType(o, DependencyObject)))
        AddHandler command.CanExecuteChanged, Sub(o, args) button.IsEnabled = command.CanExecute(GetCommandParameter(d))
        button.IsEnabled = command.CanExecute(GetCommandParameter(d))
    End Sub

    Public Shared Function GetCommand(ByVal obj As DependencyObject) As IActionCommand(Of Object)
        Return TryCast(obj.GetValue(CommandProperty), IActionCommand(Of Object))
    End Function

    Public Shared Sub SetCommand(ByVal obj As DependencyObject, ByVal value As IActionCommand(Of Object))
        obj.SetValue(CommandProperty, value)
    End Sub

    ''' <summary>
    '''     Parameter (optional)
    ''' </summary>
    Public Shared CommandParameterProperty As DependencyProperty = DependencyProperty.RegisterAttached("CommandParameter", GetType(Object), GetType(ButtonCommand), Nothing)

    Public Shared Function GetCommandParameter(ByVal obj As DependencyObject) As Object
        Return obj.GetValue(CommandParameterProperty)
    End Function

    Public Shared Sub SetCommandParameter(ByVal obj As DependencyObject, ByVal value As Object)
        obj.SetValue(CommandParameterProperty, value)
    End Sub
End Class